package org.processmining.hybridminer.algorithms.cg2hpn;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import org.deckfour.xes.model.XLog;
import org.processmining.hybridminer.models.causalgraph.HybridCausalGraph;
import org.processmining.hybridminer.models.causalgraph.HybridDirectedGraphEdge;
import org.processmining.hybridminer.models.causalgraph.HybridDirectedGraphNode;
import org.processmining.hybridminer.models.causalgraph.HybridDirectedSureGraphEdge;
import org.processmining.hybridminer.models.hybridpetrinet.Cluster;
import org.processmining.hybridminer.models.hybridpetrinet.HybridPetrinet;
import org.processmining.hybridminer.models.hybridpetrinet.PlaceEvaluation;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplaceiterator.CandidatePlaceIteratorEnum;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.BoundedInputANDOutput;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.BoundedInputPLUSOutput;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.BoundedSplitJoinOnly;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.CandidatePlaceSelectionStrategy;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.CandidatePlaceSelectionStrategyEnum;
import org.processmining.hybridminer.models.hybridpetrinet.conflictstrategy.ConflictStrategyEnum;
import org.processmining.hybridminer.plugins.HybridPNMinerSettings;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphEdge;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

/**
 * Created by demas on 19/08/16.
 */
public class CGToHybridPN {

	public static <N extends AbstractDirectedGraphNode> HybridPetrinet fuzzyCGToFuzzyPN(HybridCausalGraph graph, XLog log, HybridPNMinerSettings pNSettings) {
		long totalStartTime = System.currentTimeMillis();
		HybridPetrinet result = new HybridPetrinet("minedFuzzyPetrinet");


		// We consider only sure edges!
		Set<HybridDirectedSureGraphEdge> edges = graph.getSureEdges();

		// New parameters to be added to the graphical interface!
//		long timerSec = Integer.MAX_VALUE;
//		AtomicInteger maxPlaceNumber = new AtomicInteger(Integer.MAX_VALUE);
//		int inputPlusOutputBound = Integer.MAX_VALUE;
		//int inputPlusOutputBound = 5;
//		int inputBound = Integer.MAX_VALUE;
//		int outputBound = Integer.MAX_VALUE;
//		CandidatePlaceSelectionStrategyEnum selectionStrategyType = CandidatePlaceSelectionStrategyEnum.INPUT_PLUS_OUTPUT_BOUND;
//		CandidatePlaceIteratorEnum placeIteratorType = CandidatePlaceIteratorEnum.EDGE_NUMBER;
//		ConflictStrategyEnum conflictStrategyType = ConflictStrategyEnum.DEFAULT_CONFLICT_STRATEGY;

		// Update by sabi
		long timerSec = pNSettings.getMaxTime();
		AtomicInteger maxPlaceNumber = new AtomicInteger(pNSettings.getMaxPlaceNumber());
		int inputPlusOutputBound = pNSettings.getPlaceNumber();
		//int inputPlusOutputBound = 5;
		int inputBound = pNSettings.getIPlaceNumber();
		int outputBound = pNSettings.getOPlaceNumber();
		CandidatePlaceSelectionStrategyEnum selectionStrategyType = pNSettings.getPlaceStrategy();
		CandidatePlaceIteratorEnum placeIteratorType = pNSettings.getOrderingStrategy();
		ConflictStrategyEnum conflictStrategyType = pNSettings.getConflictStrategy();
		
		
		//ConflictStrategyEnum conflictStrategyType = ConflictStrategyEnum.NO_CONFLICT_STRATEGY;
		CandidatePlaceSelectionStrategy<HybridDirectedGraphNode> selectionStrategy = selectSelectionStrategy(selectionStrategyType, inputPlusOutputBound, inputBound, outputBound);

		// Build the clusters
		Set<Cluster<HybridDirectedSureGraphEdge, HybridDirectedGraphNode>> clusters = identifyClusters(edges, selectionStrategy, placeIteratorType, conflictStrategyType, maxPlaceNumber);


		/*
		 * MULTITHREAD
		 */
/*				ExecutorService exec = Executors.newCachedThreadPool();

				//		System.out.println("*********** Start multithread place evaluations on clusters ***********");
				//		long startTime = System.currentTimeMillis();

				for (Cluster<HybridDirectedSureGraphEdge, HybridDirectedGraphNode> c : clusters) {
					c.setLog(log);
					c.setActivityFrequencyMap(graph.getActivityFrequencyMap());
					c.setPlaceEvaluationThreshold(pNSettings.getPlaceEvalThreshold());
					c.setPrePlaceEvaluationThreshold(pNSettings.getPrePlaceEvaluationThreshold());
					exec.execute(c);
				}

				exec.shutdown();
				try {
					boolean terminated = exec.awaitTermination(timerSec, TimeUnit.SECONDS);
					if (!terminated) {
						for (Cluster<HybridDirectedSureGraphEdge, HybridDirectedGraphNode> c : clusters) {
							c.stop();
						}
					}

				} catch (InterruptedException e) {
					e.printStackTrace();
				}
*/		/*
		 * END MULTITHREAD
		 */

		/*
		 * MONOTHREAD
		 */
		ExecutorService executor = Executors.newSingleThreadExecutor();
 		for (Cluster<HybridDirectedSureGraphEdge, HybridDirectedGraphNode> c : clusters) {
			c.setLog(log);
			c.setActivityFrequencyMap(graph.getActivityFrequencyMap());
			c.setPlaceEvaluationThreshold(pNSettings.getPlaceEvalThreshold());
			c.setPrePlaceEvaluationThreshold(pNSettings.getPrePlaceEvaluationThreshold());
			// Update by sabi
			 executor.execute(c);
//			c.run();
			//c.printDotFile();
//			System.out.println(c.printStatistics());
		}
 		executor.shutdown();
 		try
 		{
			boolean terminated=executor.awaitTermination(timerSec,TimeUnit.SECONDS);
			if(!terminated)
			{
				for(Cluster<HybridDirectedSureGraphEdge,HybridDirectedGraphNode> c:clusters)
				{
					c.stop();
				}
			}
		}
 		catch(InterruptedException e)
 		{
			e.printStackTrace();
		}
		/*
		 * END MONOTHREAD
		 */	
		

		/*
		 * I have to remove places that have the same input and output sets automatically removed as I adding them to the same set
		 * and the equals has been defined as having same input and output sets.
		 */
		Set<PlaceEvaluation<HybridDirectedGraphNode>> allPlaceEvaluations = new HashSet<>();
		int maxIO = 0;

		for (Cluster<HybridDirectedSureGraphEdge, HybridDirectedGraphNode> c : clusters) {
			for (PlaceEvaluation<HybridDirectedGraphNode> pe : c.getPlacesToBeAdded()) {
				allPlaceEvaluations.add(pe);
				result.addPlaceFromPlaceEvaluation(pe);
				
				//DEBUG:
				int currentMaxIO = pe.getMaxIO();
				if (currentMaxIO > maxIO)
					maxIO = currentMaxIO;
			}
		}

//		System.out.println();
//		System.out.println("PLACES ABOVE THRESHOLD:");
//		System.out.println(allPlaceEvaluations);
//		System.out.println();
		System.out.println();
		System.out.println("maxIO : " + maxIO);


		/*
		 * Now in allPlaceEvaluation I have all the places that have to be added. Start the computation for nonRedundancy!
		 */
		//Set<PlaceEvaluation<FuzzyDirectedGraphNode>> nonRedundantPlaces = computeNonRedundantPlaces(allPlaceEvaluations);

		/*
		 * Remove places that are subset of other places. A place p is subset of q if pre(p) subseteq pre(q) AND post(p) subseteq post(q).
		 */
		//Set<PlaceEvaluation<FuzzyDirectedGraphNode>> finalPlaces = computeSubsetPlaces(nonRedundantPlaces);
		//Set<PlaceEvaluation<FuzzyDirectedGraphNode>> finalPlaces = nonRedundantPlaces;

		/*
		 * Now that I eliminated redundant places I can add the others to the net.
		 */
//		for (PlaceEvaluation<HybridDirectedGraphNode> pe : allPlaceEvaluations) {
//			result.addPlaceFromPlaceEvaluation(pe);
//			//System.out.println(pe);
//			//System.out.println(pe.evaluateReplayScore());
//		}

		/*
        Add **ALL** transitions of the causal graph to the set of transition of the petrinet.
        (Transition already present will not be added.)
		 */


		/* Then add the sure and uncertain arcs between transitions in the net coming from the causal graph
            I do not know which sure transitions have met the threshold thus have been replaced by a place transition,
             but such a check is directly in the method
		 */
		for (HybridDirectedGraphEdge edge : graph.getEdges())
			result.addTransitionsArcFromFCGEdge(edge);

		/*
		 * Finally, we need to add all the transitions of the Fuzzy Causal Graph
		 */
		for (DirectedGraphNode activity : graph.getGraph().getNodes()) {
			Transition transition = result.getTransition(activity.getLabel());
			if (transition==null)
				result.addTransition(activity.getLabel());
		}

		/*		System.out.println("********************************");
		System.out.println("Transitions : "+result.getTransitions().size());
		System.out.println("Places : "+(result.getPlaces().size()+2));
		int sureEdges = 0;
		int unsureEdges = 0;
		int f1 = 0;
		int f1_hat = 0;
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : result.getEdges()) {
			if (edge instanceof SureTransitionsArc)
				sureEdges++;
			else if (edge instanceof UncertainTransitionsArc)
				unsureEdges++;
			else
				f1++;
		}
		for (Place place : result.getPlaces()) {
			f1_hat = f1_hat+ (result.getGraph().getInEdges(place).size()*result.getGraph().getOutEdges(place).size());
		}
		System.out.println("SureEdges : "+sureEdges);
		System.out.println("Uncertain Edges : "+unsureEdges);
		System.out.println("f1 : "+(f1+2));
		System.out.println("f1 hat : "+f1_hat);
		System.out.println("********************************");*/
		long totalEndTime = System.currentTimeMillis();
		long totalElapsedTimeSec = (totalEndTime - totalStartTime)/1000;
		System.out.println("Total hybrid PN computation time: " + totalElapsedTimeSec + " sec.");

		
		return result;
	}


	private static <N extends AbstractDirectedGraphNode> CandidatePlaceSelectionStrategy<N> selectSelectionStrategy(CandidatePlaceSelectionStrategyEnum selectionStrategyType, int inputPlusOutputBound, int inputBound, int outputBound) {
		switch(selectionStrategyType) {
			case INPUT_PLUS_OUTPUT_BOUND :
				return new BoundedInputPLUSOutput<N>(inputPlusOutputBound);

			case INPUT_AND_OUTPUT_BOUND :
				return new BoundedInputANDOutput<N>(outputBound, inputBound);

			case SPLIT_JOIN_ONLY_BOUND:
				return new BoundedSplitJoinOnly<>(inputPlusOutputBound-1);

			default:
				throw new RuntimeException("Please select a valid candidate place selection strategy!");
		}

	}

	private static <N extends AbstractDirectedGraphNode> Set<PlaceEvaluation<N>> computeSubsetPlaces(Set<PlaceEvaluation<N>> allPlaces) {
		Set<PlaceEvaluation<N>> result = new HashSet<>();
		// Check the subset place by place
		for(PlaceEvaluation<N> pe : allPlaces) {
			if (!(isSubset(pe, allPlaces)))
				result.add(pe);
		}

		return result;
	}

	private static <N extends AbstractDirectedGraphNode> boolean isSubset(PlaceEvaluation<N> pe, Set<PlaceEvaluation<N>> allPlaces) {
		for (PlaceEvaluation<N> other : allPlaces) {
			if (!other.equals(pe) && other.getPlaceOutputNodes().containsAll(pe.getPlaceOutputNodes()) && other.getPlaceInputNodes().containsAll(pe.getPlaceInputNodes())) {
				System.out.println("Place Evaluation: " + pe);
				System.out.println("is subset of: " + other);
				return true;
			}
		}
		return false;
	}


	private static <N extends AbstractDirectedGraphNode> Set<PlaceEvaluation<N>> computeNonRedundantPlaces(Set<PlaceEvaluation<N>> allPlaces) {
		Set<PlaceEvaluation<N>> result = new HashSet<>();

		// Check the redundancy place by place
		for(PlaceEvaluation<N> pe : allPlaces) {
			if (!(isRedundant(pe, allPlaces)))
				result.add(pe);
		}
		return result;
	}


	private static <N extends AbstractDirectedGraphNode> boolean isRedundant(PlaceEvaluation<N> pe, Set<PlaceEvaluation<N>> allPlaces) {
		// If pe has a single inputNode or a single OutputNode cannot be redundant!
		if (pe.getPlaceInputNodes().size()==1 || pe.getPlaceOutputNodes().size()==1)
			return false;

		// Otherwise, let us compute the possible partitions of the input and those of the output:
		List<List<List<List<N>>>> outputNodePartitions = Utils.getAllPartitions(pe.getPlaceOutputNodes());
		List<List<List<List<N>>>> inputNodePartitions = Utils.getAllPartitions(pe.getPlaceInputNodes());

		// I only need to check the bijections from output to inputs which partition sets WITH THE SAME CARDINALITY!
		// Hence, I need to compute the smallest of the two List of List of ...
		int partitionsToCheck = Math.min(outputNodePartitions.size(), inputNodePartitions.size());

		//this number cannot be one, given the check on the input and output nodes...
		if (partitionsToCheck == 1)
			throw new RuntimeException("Something wrong with the isRedundant algorithm");

		// no need to check the partitions with a single element...
		for (int i=1; i<partitionsToCheck; i++) {
			/* Compute all possible bijections from outputPartition of size i+1 and inputPartition of size i+1!
			 * The bijections are obtained as follows: I leave the outputNode current partition list as it is and I compute all possible permutations
			 * of the input node partition list. The bijection always associate the first element of the first list with the first of the second. 
			 */
			List<List<List<N>>> allOutputPartitionsOfiElements = outputNodePartitions.get(i);
			List<List<List<N>>> allInputPartitionsOfiElements = inputNodePartitions.get(i);

			for (int ii=0; ii<allOutputPartitionsOfiElements.size(); ii++) {
				for (int jj=0; jj<allInputPartitionsOfiElements.size(); jj++) {
					/*
					 * WARNING! generatePerm does **SIDE EFFECT** on the argument! Perform a copy first! 
					 */
					for (List<List<N>> currentInputPerm : Utils.generatePerm(new ArrayList<List<N>>(allInputPartitionsOfiElements.get(jj)))) {
						// Now that I have a permutation, I have to check that all placeEval of the current bijection are in allPlaceEvaluation
						boolean allPresent = true;

						// I already know how many elements will be in the partition! They are i+1!
						for (int k=0; k<i+1; k++) {
							Set<N> currentOutputSet = new HashSet<N>(allOutputPartitionsOfiElements.get(ii).get(k));
							Set<N> currentInputSet = new HashSet<N>(currentInputPerm.get(k));

							// Now check if the placeEval is present!
							PlaceEvaluation<N> currentPE = new PlaceEvaluation<N>(currentOutputSet, currentInputSet, null, null, 0, 0);
							if (!allPlaces.contains(currentPE)) {
								allPresent=false;
								break;
							}
						}
						if (allPresent) {
							System.out.println("One redundant place found. OutputSet: " + pe.getPlaceOutputNodes());
							System.out.println("InputSet: " + pe.getPlaceInputNodes());
							System.out.println("It is implicated by OutputSet: " + allOutputPartitionsOfiElements.get(ii));
							System.out.println("And InputSet: " + currentInputPerm);
							return true;
						}
					}
				}
			}
		}
		return false;
	}



	// Build the clusters by least fixpoint computations, according to the definition on the paper.
	private static <E extends AbstractDirectedGraphEdge, N extends AbstractDirectedGraphNode> Set<Cluster<E, N>> identifyClusters(Set<E> edges, CandidatePlaceSelectionStrategy<N> selectionStrategy, CandidatePlaceIteratorEnum placeIteratorType, ConflictStrategyEnum conflictStrategyType, AtomicInteger maxPlaceNumber) {
		HashSet<Cluster<E, N>> clusterSet = new HashSet<>();

		// Efficiency: keep a set of edges already analyzed
		Set<E> alreadyAnalyzed = new HashSet<>();

		// for each edge not yet analyzed, create a cluster, as every edge must be contained in a cluster.
		for (E edge : edges) {
			if (alreadyAnalyzed.contains(edge))
				continue;

			// create the new set of edges constituting the new cluster
			Set<E> newCluster = new HashSet<>();

			// add the current edge to it
			newCluster.add(edge);
			alreadyAnalyzed.add(edge);
			Set<E> oldCluster = new HashSet<>();
			// add all the other edges to the current cluster.
			while (! oldCluster.equals(newCluster)) {
				// 1) oldCluster = new Cluster
				oldCluster.addAll(newCluster);

				// 2) Analyze all the edges in OldCluster and add them to newCluster...
				for (E e : oldCluster) {
					N source = (N) e.getSource();
					N target = (N) e.getTarget();

					Set<E> edgesForSource = getEdgesHavingSourceNode(source, edges);
					Set<E> edgesForTarget = getEdgesHavingTargetNode(target, edges);
					newCluster.addAll(edgesForSource);
					newCluster.addAll(edgesForTarget);

					//Add them to already analyzed, as they are already part of a cluster.
					alreadyAnalyzed.addAll(edgesForSource);
					alreadyAnalyzed.addAll(edgesForTarget);
				}
				// ...until there is no other edge to add.
			}
			// add it to the set of cluster
			clusterSet.add(new Cluster<E, N>(newCluster, selectionStrategy, placeIteratorType, conflictStrategyType, maxPlaceNumber));
		}
		return clusterSet;
	}


	/*
    Given a source node s and a set of edges, the method returns all edges having s as source.
	 */
	private static <E extends AbstractDirectedGraphEdge, N extends AbstractDirectedGraphNode> Set<E> getEdgesHavingSourceNode(N source, Set<E> edges) {
		Set<E> result = new HashSet<>();
		for (E edge : edges) {
			if (edge.getSource().equals(source))
				result.add(edge);
		}
		return result;
	}

	/*
    Given a target node t and a set of edges, the method returns all edges having t as source.
	 */
	private static <E extends AbstractDirectedGraphEdge, N extends AbstractDirectedGraphNode> Set<E> getEdgesHavingTargetNode(N target, Set<E> edges) {
		Set<E> result = new HashSet<>();
		for (E edge : edges) {
			if (edge.getTarget().equals(target))
				result.add(edge);
		}
		return result;
	}


	private static <E extends AbstractDirectedGraphEdge, N extends AbstractDirectedGraphNode> double computeIOPartitionMaxPowerSet(Set<Set<E>> sets){
		double maxPowerSet = 0;
		for (Set<E> set : sets) {
			Set<N> iNodes = new HashSet<N>();
			Set<N> oNodes = new HashSet<N>();
			for (E edge : set) {
				iNodes.add((N) edge.getTarget());
				oNodes.add((N) edge.getSource());
			}
			double numberOfReplay = Math.pow(2, iNodes.size()) * Math.pow(2,oNodes.size());
			if (numberOfReplay>maxPowerSet)
				maxPowerSet = numberOfReplay;
		}
		return maxPowerSet;
	}

	/*
	 * Prints the partition(s) set
	 */
	private static <E extends AbstractDirectedGraphEdge, N extends AbstractDirectedGraphNode> void printPartition(Set<Set<E>> edgePartition){
		Set<Set<N>> vInputNodes = new HashSet<Set<N>>();
		Set<Set<N>> vOutputNodes = new HashSet<Set<N>>();
		for (int i = 0; i < edgePartition.size(); i++) {
			Set<E> set = (Set<E>) edgePartition.toArray()[i]; 
			System.out.print("{");
			for (E e : set) {
				System.out.print(e.getSource().toString()+" "+e.getTarget().toString()+", ");
			}
			System.out.println("}");
			System.out.println("*******");
		}
	}


	private static <E extends AbstractDirectedGraphEdge> void printPartitions(Set<Set<Set<E>>> edgePartitions){
		for (Set<Set<E>> edgePartition : edgePartitions) {
			printPartition(edgePartition);
		}
	} 

}
