package org.processmining.hybridminer.dialogs;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.hybridminer.models.causalgraph.HybridCausalGraph;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplaceiterator.CandidatePlaceIteratorEnum;
import org.processmining.hybridminer.models.hybridpetrinet.candidateplacestrategy.CandidatePlaceSelectionStrategyEnum;
import org.processmining.hybridminer.models.hybridpetrinet.conflictstrategy.ConflictStrategyEnum;
import org.processmining.hybridminer.plugins.HybridPNMinerSettings;


public class HybridPNDialog extends JPanel 
{

	// Added by sabi
	JPanel strategyPanel,timeoutPanel,orderingStrategyPanel,conflictsPanel,placeNumberPanel;
	JCheckBox timeoutBox,placeoutBox;
	JComboBox placeStrategyList,orderingStrategyList,conflictsStrategy;
	JTextField timeoutField,placeNumberField,inputPlaceField,outputPlaceField,maxPlaceField;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -3587595452197538653L;
	@SuppressWarnings("unused")
	//private FuzzyCGConfiguration configuration;
	private HybridPNMinerSettings settings;

	public HybridPNDialog(UIPluginContext context, HybridCausalGraph fCG, final HybridPNMinerSettings settings)
	{
		super(new GridLayout(6,2,20,10));
		//super(new GridLayout(1,1));
		this.settings = settings;
		
		/**
		 * Additional code by sabi
		 */
		timeoutPanel=new JPanel();
		JLabel timeoutLabel=new JLabel();
		timeoutLabel.setText("<html>Time bound (minutes)</html>");
		timeoutField=new JTextField();
		timeoutField.setText(String.valueOf(Integer.MAX_VALUE));
		timeoutField.getDocument().addDocumentListener(new DocumentListener()
				{
					public void changedUpdate(DocumentEvent e){ check(); }
					public void removeUpdate(DocumentEvent e) { check(); }
					public void insertUpdate(DocumentEvent e) { check(); }
					private void check()
					{
						String value=timeoutField.getText().trim();
						if(value.trim().matches("^[0-9]*$"))
						{
//							System.out.println("Value of timeoutField contains only digits");
							if(Integer.parseInt(value.trim())>=Integer.MAX_VALUE)
							{
//								System.out.println("Value of timeout field is max");
//								timeoutBox.setSelected(true);
								settings.setMaxTime(Integer.MAX_VALUE);
							}
							else
							{
//								System.out.println("Value of timeout field is not max");
								timeoutBox.setSelected(false);
								settings.setMaxTime(Integer.parseInt(value));
							}
						}
					}
				});
		timeoutBox=new JCheckBox();
		timeoutBox.setSelected(true);
		timeoutBox.addActionListener(new ActionListener()
					{
						@Override
						public void actionPerformed(ActionEvent e)
						{
							String value=timeoutField.getText();
							if(value.trim().matches("^[0-9]*$"))
							{
								if(((JCheckBox)e.getSource()).isSelected())
								{
//									System.out.println("Max value for timeout is selected");
									timeoutField.setText(String.valueOf(Integer.MAX_VALUE));
									settings.setMaxTime(Integer.MAX_VALUE);
								}
								else
								{
//									System.out.println("Max value for timeout is not selected");
									settings.setMaxTime(Integer.parseInt(value));
								}
							}
						}
					});

		timeoutPanel.add(timeoutField);
		timeoutPanel.add(new JLabel("Max. Time"));
		timeoutPanel.add(timeoutBox);
		this.add(timeoutLabel);
		this.add(timeoutPanel);

		placeNumberPanel=new JPanel();
		JLabel placeNumberLabel=new JLabel();
		placeNumberLabel.setText("<html>Place number bound</html>");
		placeNumberField=new JTextField();
		placeNumberField.setText(String.valueOf(Integer.MAX_VALUE));
		placeNumberField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e){ check(); }
			public void removeUpdate(DocumentEvent e) { check(); }
			public void insertUpdate(DocumentEvent e) { check(); }
			private void check()
			{
				String value=placeNumberField.getText();
				if(value.trim().matches("^[0-9]*$"))
				{
//					System.out.println("Value of placeNumberField contains only digits");
					if(Integer.parseInt(value.trim())>=Integer.MAX_VALUE)
					{
//						System.out.println("Value of place number field is max");
//						placeoutBox.setSelected(true);
						settings.setMaxPlaceNumber(Integer.MAX_VALUE);
					}
					else
					{
//						System.out.println("Value of place number field is not max");
						placeoutBox.setSelected(false);
						settings.setMaxPlaceNumber(Integer.parseInt(value));
					}
				}
			}
		});
		placeoutBox=new JCheckBox();
		placeoutBox.setSelected(true);
		placeoutBox.addActionListener(new ActionListener()
				{
					@Override
					public void actionPerformed(ActionEvent e)
					{
						String value=placeNumberField.getText();
						if(value.trim().matches("^[0-9]*$"))
						{
							if(((JCheckBox)e.getSource()).isSelected())
							{
//								System.out.println("Max value for placeout is selected");
								placeNumberField.setText(String.valueOf(Integer.MAX_VALUE));
								settings.setMaxPlaceNumber(Integer.MAX_VALUE);
							}
							else
							{
//								System.out.println("Max value for placeout is not selected");
								settings.setMaxPlaceNumber(Integer.parseInt(value));
							}
						}
					}
				});
		placeNumberPanel.add(placeNumberField);
		placeNumberPanel.add(new JLabel("Max. Places"));
		placeNumberPanel.add(placeoutBox);
		this.add(placeNumberLabel);
		this.add(placeNumberPanel);
		
		
		strategyPanel=new JPanel();		
		JLabel strategyLabel=new JLabel();
		
		final JLabel maxLabel=new JLabel("Input + Output");		
		final JLabel inputLabel=new JLabel("Input");
		final JLabel outputLabel=new JLabel("Output");
		
		strategyLabel.setText("<html>Cluster strategy</html>");
		final JPanel strategyOptions=new JPanel();
		placeStrategyList=new JComboBox();
		placeStrategyList.addItem("Set Input + Output");
		placeStrategyList.setSelectedIndex(0);
		placeStrategyList.addItem("Set Input and set Output");
		placeStrategyList.addItem("Join Split");
		placeStrategyList.addActionListener(new ActionListener()
					{
						@Override
						public void actionPerformed(ActionEvent e)
						{
							String value=String.valueOf(((JComboBox)e.getSource()).getSelectedItem());
							System.out.println("You have selected "+value);
							switch(((JComboBox)e.getSource()).getSelectedIndex())
							{
								case 0:
									// Enable/disable the different fields, depending on whether they are needed or not and select the strategy
									maxPlaceField.setEnabled(true);
									inputPlaceField.setEnabled(false);
									outputPlaceField.setEnabled(false);
									maxLabel.setEnabled(true);
									inputLabel.setEnabled(false);
									outputLabel.setEnabled(false);
									settings.setPlaceStrategy(CandidatePlaceSelectionStrategyEnum.INPUT_PLUS_OUTPUT_BOUND);
									
									break;
								
								case 1:
									// Enable/disable the different fields, depending on whether they are needed or not and select the strategy
									maxPlaceField.setEnabled(false);
									inputPlaceField.setEnabled(true);
									outputPlaceField.setEnabled(true);
									maxLabel.setEnabled(false);
									inputLabel.setEnabled(true);
									outputLabel.setEnabled(true);
									settings.setPlaceStrategy(CandidatePlaceSelectionStrategyEnum.INPUT_AND_OUTPUT_BOUND);
									
									break;
								
								case 2:
									// Enable/disable the different fields, depending on whether they are needed or not and select the strategy
									maxPlaceField.setEnabled(true);
									inputPlaceField.setEnabled(false);
									outputPlaceField.setEnabled(false);
									maxLabel.setEnabled(true);
									inputLabel.setEnabled(false);
									outputLabel.setEnabled(false);
									settings.setPlaceStrategy(CandidatePlaceSelectionStrategyEnum.SPLIT_JOIN_ONLY_BOUND);

									break;
								
								default:
									// Default case if value is different that the expected one. Handle problem
									break;
							}
						}
					});
		strategyOptions.setLayout(new GridLayout(3,2));
		
		// Create JTextField for setting the maximum amount of places. It is enabled, since this option is the one selected by default
		maxPlaceField=new JTextField();
		maxPlaceField.setText(String.valueOf(Integer.MAX_VALUE));
		maxPlaceField.setEnabled(true);
		maxPlaceField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e){ check(); }
			public void removeUpdate(DocumentEvent e) { check(); }
			public void insertUpdate(DocumentEvent e) { check(); }
			private void check()
			{
				String value=maxPlaceField.getText();
				if(value.trim().matches("^[0-9]*$"))
				{
					settings.setPlaceNumber(Integer.parseInt(value));
				}
			}
		});
//		JLabel maxLabel=new JLabel("Input + Output");
		strategyOptions.add(maxLabel);
		strategyOptions.add(maxPlaceField);
		
//		JLabel inputLabel=new JLabel("Input");
//		JLabel outputLabel=new JLabel("Output");
		inputLabel.setEnabled(false);
		outputLabel.setEnabled(false);
		strategyOptions.add(inputLabel);
		strategyOptions.add(outputLabel);
		
		// Create JTextField for setting the maximum amount of input places. It is disabled, since this option is not the one selected by default
		inputPlaceField=new JTextField();
		inputPlaceField.setVisible(true);
		inputPlaceField.setEnabled(false);
		inputPlaceField.setText(String.valueOf(Integer.MAX_VALUE));
		inputPlaceField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e){ check(); }
			public void removeUpdate(DocumentEvent e) { check(); }
			public void insertUpdate(DocumentEvent e) { check(); }
			private void check()
			{
				String valueInput=inputPlaceField.getText();
				String valueOutput=outputPlaceField.getText();
				if(valueInput.trim().matches("^[0-9]*$") && valueOutput.trim().matches("^[0-9]*$"))
				{
					settings.setPlaceNumber(Integer.parseInt(valueInput),Integer.parseInt(valueOutput));
				}
			}
		});
		strategyOptions.add(inputPlaceField);
		
		// Create JTextField for setting the maximum amount of output places. It is disabled, since this option is not the one selected by default
		outputPlaceField=new JTextField();
		outputPlaceField.setVisible(true);
		outputPlaceField.setEnabled(false);
		outputPlaceField.setText(String.valueOf(Integer.MAX_VALUE));
		outputPlaceField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e){ check(); }
			public void removeUpdate(DocumentEvent e) { check(); }
			public void insertUpdate(DocumentEvent e) { check(); }
			private void check()
			{
				String valueInput=inputPlaceField.getText();
				String valueOutput=outputPlaceField.getText();
				if(valueInput.trim().matches("^[0-9]*$") && valueOutput.trim().matches("^[0-9]*$"))
				{
					settings.setPlaceNumber(Integer.parseInt(valueInput),Integer.parseInt(valueOutput));
				}
			}
		});
		strategyOptions.add(outputPlaceField);
		
		// Set default strategy
		settings.setPlaceStrategy(CandidatePlaceSelectionStrategyEnum.INPUT_PLUS_OUTPUT_BOUND);
		settings.setPlaceNumber(Integer.MAX_VALUE);
		settings.setPlaceNumber(Integer.MAX_VALUE,Integer.MAX_VALUE);
		
		// Add everything to the JPanel that is going to be displayed to the user
		strategyPanel.add(placeStrategyList);
		strategyPanel.add(strategyOptions);
		this.add(strategyLabel);
		this.add(strategyPanel);
		
		
		orderingStrategyPanel=new JPanel();
		JLabel orderingStrategyLabel=new JLabel();
		orderingStrategyLabel.setText("<html>Place ordering strategy</html>");
		orderingStrategyList=new JComboBox();
		orderingStrategyList.addItem("Edge number");
		orderingStrategyList.addItem("Edge number (optimised)");
		orderingStrategyList.setSelectedIndex(1);
		orderingStrategyList.addItem("Output then input number");
		orderingStrategyList.addActionListener(new ActionListener()
				{
					@Override
					public void actionPerformed(ActionEvent e)
					{
						String value=String.valueOf(((JComboBox)e.getSource()).getSelectedItem());
						switch(((JComboBox)e.getSource()).getSelectedIndex())
						{
							case 0:
//								System.out.println("You have selected strategy 1");
								settings.setOrderingStrategy(CandidatePlaceIteratorEnum.EDGE_NUMBER);
								break;
							case 1:
//								System.out.println("You have selected strategy 2");
								settings.setOrderingStrategy(CandidatePlaceIteratorEnum.EDGE_NUMBER_OPT);
								break;
							case 2:
//								System.out.println("You have selected strategy 3");
								settings.setOrderingStrategy(CandidatePlaceIteratorEnum.OUTPUT_THEN_INPUT_NUMBER);
								break;
							default: 
								System.out.println("You have selected no strategy....how's that even possible?");
								break;
						}
					}
				});

		orderingStrategyPanel.add(orderingStrategyList);
		this.add(orderingStrategyLabel);
		this.add(orderingStrategyPanel);
		
		
		conflictsPanel=new JPanel();		
		JLabel conflictsLabel=new JLabel();		
		conflictsLabel.setText("<html>Conflict strategy</html>");
		conflictsStrategy=new JComboBox();
		conflictsStrategy.addItem("Default conflict strategy");
		conflictsStrategy.addItem("Default conflict strategy (optimised)");
		conflictsStrategy.setSelectedIndex(1);
		conflictsStrategy.addItem("No conflict strategy");
		conflictsStrategy.addActionListener(new ActionListener()
					{
						@Override
						public void actionPerformed(ActionEvent e)
						{
							String value=String.valueOf(((JComboBox)e.getSource()).getSelectedItem());
							switch(value)
							{
								case "Conflict 1":
//									System.out.println("You have selected conflict 1");
									settings.setConflictStrategy(ConflictStrategyEnum.DEFAULT_CONFLICT_STRATEGY);
									break;
								case "Conflict 2":
//									System.out.println("You have selected conflict 2");
									settings.setConflictStrategy(ConflictStrategyEnum.DEFAULT_CONFLICT_STRATEGY_OPT);
									break;
								case "Conflict 3":
//									System.out.println("You have selected conflict 3");
									settings.setConflictStrategy(ConflictStrategyEnum.NO_CONFLICT_STRATEGY);
									break;
								default: 
									System.out.println("You have selected no conflict....how's that even possible?");
									break;
							}
						}
					});
		
		
//		conflictsPanel.add(conflictsLabel);
		conflictsPanel.add(conflictsStrategy);
		this.add(conflictsLabel);
		this.add(conflictsPanel);
				
		
		
		Component space = Box.createHorizontalStrut(10);
		//this.add(space);
		
		//final JPanel label1 = new JPanel(new GridLayout(2,1,1,-10));
		final JLabel placeEvaluationThresholdLabel = new JLabel();
		placeEvaluationThresholdLabel.setText("<html>Required Place Quality <br> (Minimal fraction of fitting traces <br>considering only the traces that activate the place)</html>");
		this.add(placeEvaluationThresholdLabel);

/*		final JLabel placeEvaluationThresholdLabel2 = new JLabel();
		placeEvaluationThresholdLabel2.setText("(Minimal fraction of fitting traces considering only the traces that activate the place)");
		label1.add(placeEvaluationThresholdLabel2);*/
		
/*		final JLabel placeEvaluationThresholdLabel3 = new JLabel();
		placeEvaluationThresholdLabel3.setText("");
		label1.add(placeEvaluationThresholdLabel3);*/
		
		//this.add(label1);
		
		//space = Box.createHorizontalStrut(1);
		//this.add(space);
		
		final JSlider placeEvaluationThresholdSlider = new JSlider();
		placeEvaluationThresholdSlider.setMinimum(0);
		placeEvaluationThresholdSlider.setMaximum(100);
		
		placeEvaluationThresholdSlider.setMajorTickSpacing(50);
		placeEvaluationThresholdSlider.setMinorTickSpacing(10);
		placeEvaluationThresholdSlider.setPaintTicks(true);
		placeEvaluationThresholdSlider.setValue(80);
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		labelTable.put(0, new JLabel("0%"));
		labelTable.put(50, new JLabel("50%"));
		labelTable.put(100, new JLabel("100%"));
		placeEvaluationThresholdSlider.setLabelTable(labelTable);
		placeEvaluationThresholdSlider.setPaintLabels(true);
		this.add(placeEvaluationThresholdSlider);
		
//		int intValue = (int) (settings.getPlaceEvalThreshold()*100);
//		placeEvaluationThresholdSlider.setValue(intValue);
		placeEvaluationThresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				settings.setPlaceEvalThreshold(placeEvaluationThresholdSlider.getValue()/100.0);
			}
		});
		this.add(placeEvaluationThresholdSlider);
		
		
		
		//space = Box.createVerticalStrut(20);
		//this.add(space);
		
		// Removed by sabi

		/*
		 *  ******************** MAX CLUSTER SIZE ENABLED **************************************************
		 */
		
		/*space = Box.createVerticalStrut(10);
		this.add(space);*/
		
		/*
		//final JPanel label2 = new JPanel(new GridLayout(1,1));
		final JLabel maxClusterSizeEnabled = new JLabel();
		maxClusterSizeEnabled.setText("Maximum Cluster Size Enabled");
		this.add(maxClusterSizeEnabled);
		//label2.add(maxClusterSizeEnabled);
		//this.add(label2);

		//space = Box.createHorizontalStrut(1);
		//this.add(space);

		final JCheckBox mCSizeCbx = new JCheckBox();
		mCSizeCbx.setBackground(new Color(150,150,150));
		mCSizeCbx.setHorizontalAlignment(SwingConstants.CENTER);
		mCSizeCbx.setSelected(settings.isMaxClusterSizeEnabled());
		mCSizeCbx.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				settings.setMaxClusterSizeEnabled(mCSizeCbx.isSelected());
			}
		});
		this.add(mCSizeCbx);
		
		*/
		
		/*
		 *  ******************** MAX CLUSTER SIZE **************************************************
		 */
//		
//		final JLabel maxClusterSizeLabel = new JLabel();
//		maxClusterSizeLabel.setText("Max size of clusters");
//		this.add(maxClusterSizeLabel);
//
//		space = Box.createHorizontalStrut(10);
//		this.add(space);
//	
//
//		final JSlider maxClusterSizeSlider = new JSlider();
//		
//		maxClusterSizeSlider.setMinimum(0);
//		maxClusterSizeSlider.setMaximum(50);
//		
//		intValue = settings.getMaxClusterSize();
//		maxClusterSizeSlider.setValue(intValue);
//		maxClusterSizeSlider.addChangeListener(new ChangeListener() {
//			public void stateChanged(ChangeEvent e) {
//				settings.setMaxClusterSize(maxClusterSizeSlider.getValue());
//			}
//		});
//		maxClusterSizeSlider.setMajorTickSpacing(10);
//		maxClusterSizeSlider.setMinorTickSpacing(5);
//		maxClusterSizeSlider.setPaintTicks(true);
//		labelTable = new Hashtable<Integer, JLabel>();
//		labelTable.put(0, new JLabel("0"));
//		labelTable.put(25, new JLabel("25"));
//		labelTable.put(50, new JLabel("50"));
//		maxClusterSizeSlider.setLabelTable(labelTable);
//		maxClusterSizeSlider.setPaintLabels(true);
//		this.add(maxClusterSizeSlider);
//		
//		space = Box.createVerticalStrut(20);
//		this.add(space);
		
		/*
		 *  ******************** END MAX CLUSTER SIZE **************************************************
		 */
		

		/*final JLabel allConnected = new JLabel();
		allConnected.setText("All tasks connected");
		this.add(allConnected);

		space = Box.createHorizontalStrut(10);
		this.add(space);

		JCheckBox connCbx = new JCheckBox();
		connCbx.setBackground(Color.GRAY);
		connCbx.setSelected(settings.getHmSettings().isUseAllConnectedHeuristics());
		connCbx.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				HeuristicsMinerSettings hMS = settings.getHmSettings();
				//hMS.setUseAllConnectedHeuristics(connCbx.isSelected());
				settings.setHmSettings(hMS);
			}
		});
		this.add(connCbx);*/
	}
	

}
