package org.processmining.hybridminer.models.causalgraph;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.hybridminer.plugins.HybridCGMinerSettings;
import org.processmining.models.heuristics.impl.ActivitiesMappingStructures;
import org.processmining.plugins.heuristicsnet.miner.heuristics.HeuristicsMetrics;

/**
 * Created by demas on 27/07/16.
 */
public class HybridCausalGraph extends CausalGraph<HybridDirectedGraphNode, HybridDirectedGraphEdge> {
	private Map<String, HybridDirectedGraphNode> labelNodeMap;
	private Map<String, Integer> activityFrequencyMap;
    protected List<Double> rowSumDirectDependency;
    protected List<Double> columnSumDirectDependency;

	
	private XLog log;
	private ActivitiesMappingStructures activitiesMappingStructures;
	private HeuristicsMetrics metrics;
	private HybridCGMinerSettings settings;
	
	public HybridCausalGraph() {
		super();
		this.labelNodeMap = new HashMap<>();
		this.activityFrequencyMap = new HashMap<>();
		this.log = null;
		this.rowSumDirectDependency = new ArrayList<Double>();
		this.columnSumDirectDependency = new ArrayList<Double>();
	}
	
	public Map<String, Integer> getActivityFrequencyMap() {
		return activityFrequencyMap;
	}

	public void setActivityFrequencyMap(Map<String, Integer> activityFrequencyMap) {
		this.activityFrequencyMap = activityFrequencyMap;
	}
	
	public XLog getLog() {
		return log;
	}

	public void setLog(XLog log) {
		this.log = log;
	}

	public HybridCGMinerSettings getSettings() {
		return settings;
	}

	public void setSettings(HybridCGMinerSettings settings) {
		this.settings = settings;
	}
	
	public ActivitiesMappingStructures getActivitiesMappingStructures() {
		return activitiesMappingStructures;
	}

	public void setActivitiesMappingStructures(ActivitiesMappingStructures activitiesMappingStructures) {
		this.activitiesMappingStructures = activitiesMappingStructures;
	}

	public HeuristicsMetrics getMetrics() {
		return metrics;
	}

	public void setMetrics(HeuristicsMetrics metrics) {
		this.metrics = metrics;
	}

	public void setRowSumDirectDependency(List<Double> rowSumDirectDependency) {
		this.rowSumDirectDependency = rowSumDirectDependency;
	}
	
	public Double getRowSumDirectDependency(int row) {
		return rowSumDirectDependency.get(row);
	}

	public void setRowSumDirectDependency(int row, double value) {
		this.rowSumDirectDependency.set(row, value);
	}
	
	public void setColumnSumDirectDependency(List<Double> columnSumDirectDependency) {
		this.columnSumDirectDependency = columnSumDirectDependency;
	}
	
	public Double getColumnSumDirectDependency(int column) {
		return columnSumDirectDependency.get(column);
	}

	public void setColumnSumDirectDependency(int column, double value) {
		this.columnSumDirectDependency.set(column, value);
	}

	/**
	 * We build fuzzyCausalGraph with a strong limitation: we do not ever have two nodes with the same
	 * label.
	 * @param nodeLabel the node to be added
	 * @return a new node with the specified label if there is no other node in the graph with the same label,
	 * otherwise the already existing node.
	 */
	public HybridDirectedGraphNode addNode(String nodeLabel) {
		HybridDirectedGraphNode alreadyPresent = this.labelNodeMap.get(nodeLabel);
		if (alreadyPresent == null) {
			HybridDirectedGraphNode node = new HybridDirectedGraphNode(this, nodeLabel);
			super.addNode(node);
			labelNodeMap.put(node.getLabel(), node);
			return node;
		}
		else
			return alreadyPresent;
	}
	
	// Update by sabi (changed return type from void to HybridDirectedSureGraphEdge)
	public HybridDirectedSureGraphEdge addSureEdge(HybridDirectedGraphNode sourceNode, HybridDirectedGraphNode targetNode, double value1, double value2){
		HybridDirectedSureGraphEdge sureEdge = new HybridDirectedSureGraphEdge(sourceNode, targetNode, value1, value2);
//		HybridDirectedSureGraphEdge sureEdge=new HybridDirectedSureGraphEdge(sourceNode, targetNode);
//		sureEdge.setLabel(value1);
//		System.out.println("Sure edge: "+sureEdge);
		super.addEdge(sureEdge);
		try
		{
			this.wait();
		}
		catch(Exception e){}
		return sureEdge;
	}
	
	public HybridDirectedUncertainGraphEdge addUncertainEdge(HybridDirectedGraphNode sourceNode, HybridDirectedGraphNode targetNode, double value1, double value2){
		HybridDirectedUncertainGraphEdge unsureEdge = new HybridDirectedUncertainGraphEdge(sourceNode, targetNode, value1, value2);
//		HybridDirectedUncertainGraphEdge unsureEdge = new HybridDirectedUncertainGraphEdge(sourceNode, targetNode);
//		unsureEdge.setLabel(value1);
		super.addEdge(unsureEdge);
		try
		{
			this.wait();
		}
		catch(Exception e){}
		return unsureEdge;
	}


	public HybridDirectedGraphNode getNode(String label){
		return this.labelNodeMap.get(label);
	}

	public Set<HybridDirectedSureGraphEdge> getSureEdges() {
		HashSet<HybridDirectedSureGraphEdge> result = new HashSet<>();
		for (HybridDirectedGraphEdge edge : this.getEdges()) {
			if (edge instanceof HybridDirectedSureGraphEdge)
				result.add((HybridDirectedSureGraphEdge)edge);
		}
		return result;
	}

	
	@Override
	protected Object clone() throws CloneNotSupportedException {
		HybridCausalGraph newFCG = new HybridCausalGraph();
		for (HybridDirectedGraphNode node : this.getNodes()) {
			newFCG.addNode(node);
		}
	 	for (HybridDirectedGraphEdge edge : this.getEdges()) {
			newFCG.addEdge(edge);
		}
		return newFCG;
	}

	
	/**
	 * It empties the graph associated to this fCG
	 */
	public void emptyGraph() {
		HybridCausalGraph clonedGraph;
		try {
			clonedGraph = (HybridCausalGraph) this.clone();
			for (HybridDirectedGraphNode node : clonedGraph.getNodes()) {
				this.removeNode(node);
			}
			this.labelNodeMap = new HashMap<>();
		} catch (CloneNotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	

	@Override
	public String toString() {
		String graphString = "*** GRAPH *** "+this.getLabel()+"\n";
        Set<HybridDirectedGraphNode> nodes  = this.getNodes();
        graphString+= "** NODES **\n";
        for (HybridDirectedGraphNode node : nodes) {
        	graphString+= node.getId()+" "+node.getLabel()+"\n";
		}
        graphString+= "** EDGES **\n";
        Set<HybridDirectedGraphEdge> edges = this.getEdges();
        for (HybridDirectedGraphEdge edge : edges) {
        	if (edge instanceof HybridDirectedSureGraphEdge)
        		graphString+= "SURE EDGE "+edge.getSource().getLabel()+" "+edge.getTarget().getLabel()+" "+edge.getLabel()+"\n";
        	else
        		graphString+= "UNSURE EDGE "+edge.getSource().getLabel()+" "+edge.getTarget().getLabel()+" "+edge.getLabel()+"\n";
			
		}
        return graphString;
	}

	
	
}
