package org.processmining.hybridminer.models.hybridpetrinet;

import java.util.Collection;
import java.util.Set;

import org.processmining.hybridminer.algorithms.cg2hpn.Utils;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;

public class PartialPlaceEvaluation<N extends AbstractDirectedGraphNode>  {
	private Set<N> placeOutputNodes, placeInputNodes;



	public PartialPlaceEvaluation(Set<N> placeOutputNodes, Set<N> placeInputNodes) {
		this.placeOutputNodes = placeOutputNodes;
		this.placeInputNodes = placeInputNodes;
	}


	public int getMaxIO() {
		return Math.max(this.placeOutputNodes.size(), this.placeInputNodes.size());
	}


	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((placeInputNodes == null) ? 0 : placeInputNodes.hashCode());
		result = prime * result + ((placeOutputNodes == null) ? 0 : placeOutputNodes.hashCode());
		return result;
	}



	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		PartialPlaceEvaluation other = (PartialPlaceEvaluation) obj;
		if (placeInputNodes == null) {
			if (other.placeInputNodes != null)
				return false;
		} else if (!placeInputNodes.equals(other.placeInputNodes))
			return false;
		if (placeOutputNodes == null) {
			if (other.placeOutputNodes != null)
				return false;
		} else if (!placeOutputNodes.equals(other.placeOutputNodes))
			return false;
		return true;
	}

	
	/*
	 * Check if this is disjoint with places.
	 */
	public boolean isDisjoint(Collection<PlaceEvaluation<N>> places) {
		// for each outputPlace
		for (N n : this.placeOutputNodes) {
			for (PartialPlaceEvaluation<N> p : places) {
				if (p.getPlaceOutputNodes().contains(n))
					return false;
			}
		}

		// for each inputPlace
		for (N n : this.placeInputNodes) {
			for (PartialPlaceEvaluation<N> p : places) {
				if (p.getPlaceInputNodes().contains(n))
					return false;
			}
		}
		return true;
	}
	
	
    /*
	 * Given a set of (Partial)PlaceEvaluations, it checks if they are all pairwise disjoint.
	 */
	public static <N extends AbstractDirectedGraphNode> boolean areDisjoint(Collection<PlaceEvaluation<N>> places) {
		for (PartialPlaceEvaluation<N> ppei : places) {
			for (PartialPlaceEvaluation<N> ppej : places) {
				Set<N> ppeiOutPlaceClone = Utils.copySet(ppei.getPlaceOutputNodes());

				if (ppeiOutPlaceClone.retainAll(ppej.getPlaceOutputNodes())) 
					return false;

				Set<N> ppeiInputPlaceClone = Utils.copySet(ppei.getPlaceInputNodes());

				if (ppeiInputPlaceClone.retainAll(ppej.getPlaceInputNodes()))
					return false;
			}
		}
		return true;
	}



	public Set<N> getPlaceOutputNodes() {
		return placeOutputNodes;
	}



	public Set<N> getPlaceInputNodes() {
		return placeInputNodes;
	}


	/*
	 * Given a place p, Checks if the inputTransitions *p are a subset (even not proper) of the outputTransitions p*.
	 */
	public boolean isSyphon() {
		if (this.placeInputNodes.containsAll(this.placeOutputNodes))
			return true;
		return false;
	}



}