package org.processmining.hybridminer.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.hybridminer.algorithms.preprocessing.LogFilterer;
import org.processmining.hybridminer.algorithms.preprocessing.LogPreprocessor;
import org.processmining.hybridminer.dialogs.CGDialog;
import org.processmining.hybridminer.models.causalgraph.HybridCausalGraph;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;

/**
 * Created by demas on 25/07/16.
 */

@Plugin(name = "Hybrid Causal Graph Miner", parameterLabels = {"log", "Hybrid Causal Graph Configuration" }, 
	    returnLabels = {"Causal Graph"}, returnTypes = {HybridCausalGraph.class})
public class HybridCGMinerPlugin {

	private HybridCausalGraph privateFCGMinerPlugin(PluginContext context, XLog log, HybridCGMinerSettings settings) {
        //double t1 = System.currentTimeMillis();
		XLog preprocessedLog = LogPreprocessor.preprocessLog(log);
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(preprocessedLog, settings.getHmSettings().getClassifier());
		System.out.println("**** PREPROCESSING OVER ******");
		XLog filteredLog = LogFilterer.filterLogByActivityFrequency(preprocessedLog, logInfo, settings);
		System.out.println("**** FILTERING OVER ******");
		HybridCGMiner miner = new HybridCGMiner(filteredLog, filteredLog.getInfo(settings.getHmSettings().getClassifier()), settings);
		HybridCausalGraph fCG = miner.mineFCG(settings);
/*        double t2 = System.currentTimeMillis();
		System.out.println("*****************************************");
		System.out.println("TIME REQUIRED FOR FCG "+(t2 - t1));
		System.out.println("*****************************************");*/
		return fCG;

	}
	
	/**
	 * The plug-in variant that runs in any context and requires a configuration.
	 */ 
	@UITopiaVariant(affiliation = "FBK", author = "R. De Masellis et al.", email = "r.demasellis|dfmchiara@fbk.eu")
	@PluginVariant(variantLabel = "HybridCGMiner, parameters", requiredParameterLabels = { 0, 1})
	public HybridCausalGraph configuredFPNMinerPlugin(PluginContext context, XLog log, HybridCGMinerSettings settings) {
		return privateFCGMinerPlugin(context, log, settings);
	}
	
	/**
	 * The plug-in variant that runs in any context and uses the default configuration.
	 */
	@UITopiaVariant(affiliation = "FBK", author = "R. De Masellis et al.", email = "r.demasellis|dfmchiara@fbk.eu")
	@PluginVariant(variantLabel = "HybridCGMiner, parameters", requiredParameterLabels = { 0 })
	public HybridCausalGraph defaultFCGMinerPlugin(PluginContext context, XLog log) {
		// Get the default configuration.
		XEventClassifier nameCl = new XEventNameClassifier();
		HeuristicsMinerSettings hMS = new HeuristicsMinerSettings();
		hMS.setClassifier(nameCl);
					
		HybridCGMinerSettings settings = new HybridCGMinerSettings(hMS, log);
		//settings.setHmSettings(hMS);
		// Do the heavy lifting.
	    return privateFCGMinerPlugin(context, log, settings);
	}
	
	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get the configuration.
	 */
	@UITopiaVariant(affiliation = "FBK", author = "R. De Masellis et al.", email = "r.demasellis|dfmchiara@fbk.eu")
	@PluginVariant(variantLabel = "HybridCGMiner, dialog", requiredParameterLabels = { 0 })
	public HybridCausalGraph dialogFPNMinerPlugin(UIPluginContext context, XLog log) {
		// Get the default configuration.
	    
		XEventClassifier nameCl = new XEventNameClassifier();
		HeuristicsMinerSettings hMS = new HeuristicsMinerSettings();
		hMS.setClassifier(nameCl);  
		
	    HybridCGMinerSettings settings = new HybridCGMinerSettings(hMS,log);
		//settings.setHmSettings(hMS);
		
	    // Get a dialog for this configuration.
	    CGDialog dialog = new CGDialog(context, log, settings);
	    // Show the dialog. User can now change the configuration.
	    InteractionResult result = context.showWizard("Causal Graph Settings", true, true, dialog);
	    // User has close the dialog.
	    if (result == InteractionResult.FINISHED) {
			// Do the heavy lifting.
	    	return privateFCGMinerPlugin(context, log, settings);
	    }
	    // Dialog got canceled.
	    return null;
	}	
}
