package org.processmining.hybridminer.plugins;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;

/**
 * Created by demas on 25/07/16.
 */
public class HybridCGMinerSettings {
	/*
	 * NB DECIDERE SE AGGIUNGERE ANCHE USEALLCONNECTEDHEURISTICS
	 */
	
	

	// Settings of the original HeuristicsMiner
    private HeuristicsMinerSettings hmSettings;
    
    private int maxOccurrence;
    private int positiveObservationThreshold;
    private boolean useAllConnectedHeuristics;
   
    // Threshold between 0 and 1 above which the causality is considered certain.
    private double sureThreshold;

    /*
        Threshold between 0 and 1 above which the causality is considered uncertain.
        Notice that questionMarkThreshold must be obviously < Threshold.
    */
    private double questionMarkThreshold;
    
    
    private double causalityWeight;
    
    
    private static double POSITIVEOBSERVATIONDEGREE = 0.3;
    private static double SURETHRESHOLD = 0.6;
    private static double QUESTIONMARKTHRESHOLD = 0.5;
    private static double CAUSALITYWEIGHT = 0.5;
    
    private Map<String, Integer> activityFrequencyMap;

    public HybridCGMinerSettings(HeuristicsMinerSettings hms, int positiveObservationThreshold, 
    		double sureThreshold, double questionMarkThreshold, double causalityWeight) {
        this.hmSettings = hms;
        this.positiveObservationThreshold = positiveObservationThreshold;
        this.sureThreshold = sureThreshold;
        this.questionMarkThreshold = questionMarkThreshold;
        this.activityFrequencyMap = new HashMap<>();
        this.causalityWeight = causalityWeight;

    }
    
    //default one
    public HybridCGMinerSettings(HeuristicsMinerSettings hms,
    		double sureThreshold, double questionMarkThreshold, double causalityWeight) {
        this.hmSettings = hms;
        this.sureThreshold = sureThreshold;
        this.questionMarkThreshold = questionMarkThreshold;
        this.activityFrequencyMap = new HashMap<>();
        this.causalityWeight = causalityWeight;

    }
    
    public HybridCGMinerSettings(HeuristicsMinerSettings hms, XLog log){
    	this(hms, SURETHRESHOLD, QUESTIONMARKTHRESHOLD, CAUSALITYWEIGHT);
    	this.maxOccurrence = computeMaxOccurrence(hms, log);
        this.positiveObservationThreshold = (int) (POSITIVEOBSERVATIONDEGREE*this.maxOccurrence);
    }
    
    public double getCausalityWeight() {
		return causalityWeight;
	}

	public void setCausalityWeight(double causalityWeight) {
		this.causalityWeight = causalityWeight;
	}

	public Map<String, Integer> getActivityFrequencyMap() {
    	return activityFrequencyMap;
    }
    
    public void setActivityFrequencyMap(Map<String, Integer> activityFrequencyMap) {
    	this.activityFrequencyMap = activityFrequencyMap;
    }


    public HeuristicsMinerSettings getHmSettings() {
        return hmSettings;
    }


	
/*	public void setHmSettings(HeuristicsMinerSettings hmSettings) {
		this.hmSettings = hmSettings;
	}*/
	
	public boolean isUseAllConnectedHeuristics() {
		return useAllConnectedHeuristics;
	}

	public void setUseAllConnectedHeuristics(boolean useAllConnectedHeuristics) {
		this.useAllConnectedHeuristics = useAllConnectedHeuristics;
	}

	public int getPositiveObservationThreshold() {
		return positiveObservationThreshold;
	}

	public void setPositiveObservationThreshold(int positiveObservationThreshold) {
		this.positiveObservationThreshold = positiveObservationThreshold;
	}
	
    public double getSureThreshold() {
        return sureThreshold;
    }

    public double getQuestionMarkThreshold() {
        return questionMarkThreshold;
    }

    
	public void setSureThreshold(double sureThreshold) {
		this.sureThreshold = sureThreshold;
	}

	public void setQuestionMarkThreshold(double questionMarkThreshold) {
		this.questionMarkThreshold = questionMarkThreshold;
	}
	

	public int getMaxOccurrence() {
		return maxOccurrence;
	}

	public void setMaxOccurrence(int maxOccurrence) {
		this.maxOccurrence = maxOccurrence;
	}


	
	private int computeMaxOccurrence(HeuristicsMinerSettings hms, XLog log) {
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, hms.getClassifier());
	
		Map<String, Integer> activityFrequencyMap = new HashMap<String, Integer>();
		int logEventOccurrences = log.size();
		for (XTrace trace : log) {
			Set<String> traceEvents = new HashSet<String>();
			
	        for (XEvent event : trace) {
	            String eventKey = logInfo.getEventClasses(hms.getClassifier()).getClassOf(event).getId();
	            traceEvents.add(eventKey);
	        }
	        for (String eventKey : traceEvents) {				
	            Integer value = activityFrequencyMap.get(eventKey);
	            if (value==null)
	            	value = new Integer(1);
	            else 
	            	value = value+1;
	            activityFrequencyMap.put(eventKey, value);

			}
		}
		
		
		Integer max = Collections.max(activityFrequencyMap.values());
		if (max<logEventOccurrences)
			max=logEventOccurrences;
		return max;
	}

	
	@Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        HybridCGMinerSettings that = (HybridCGMinerSettings) o;
        if (Integer.compare(that.getPositiveObservationThreshold(), getPositiveObservationThreshold()) != 0) return false;
        if (Double.compare(that.getSureThreshold(), getSureThreshold()) != 0) return false;
        if (Double.compare(that.getQuestionMarkThreshold(), getQuestionMarkThreshold()) != 0) return false;
        if (Double.compare(that.getCausalityWeight(), getCausalityWeight()) != 0) return false;
        return getHmSettings().equals(that.getHmSettings());

        
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        result = getHmSettings().hashCode();
        temp = getPositiveObservationThreshold();
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getSureThreshold());
        result = 31 * result + (int) (temp ^ (temp >>> 32));    
        temp = Double.doubleToLongBits(getQuestionMarkThreshold());
        result = 31 * result + (int) (temp ^ (temp >>> 32));  
        temp = Double.doubleToLongBits(getCausalityWeight());
        result = 31 * result + (int) (temp ^ (temp >>> 32));          
        return result;
    }
}
